#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/8/20 14:41
Desc: 鸡蛋价格
https://www.jidan7.com/trend/
"""
import re

import pandas as pd
import requests
from bs4 import BeautifulSoup


def futures_egg_price_yearly() -> pd.DataFrame:
    """
    各年度产区鸡蛋价格走势
    https://www.jidan7.com/trend/
    :return: 各年度产区鸡蛋价格走势
    :rtype: pandas.DataFrame
    """
    url = "https://www.jidan7.com/trend/"
    r = requests.get(url)
    soup = BeautifulSoup(r.text, "lxml")
    js_text = soup.find_all("script")[7].string
    js_text_processed = js_text.replace("\r\n", "")
    js_text_processed = re.findall(r"(\[.*?])", js_text_processed)
    year_list = eval(js_text_processed[1])
    date_list = eval(js_text_processed[2])
    value_2015_list = eval(js_text_processed[4])
    value_2016_list = eval(js_text_processed[6])
    value_2017_list = eval(js_text_processed[8])
    value_2018_list = eval(js_text_processed[10])
    value_2019_list = eval(js_text_processed[12])
    value_2020_list = eval(js_text_processed[14])
    value_2021_list = eval(js_text_processed[16])
    temp_df = pd.DataFrame(
        [
            date_list,
            value_2015_list,
            value_2016_list,
            value_2017_list,
            value_2018_list,
            value_2019_list,
            value_2020_list,
            value_2021_list,
        ]
    ).T
    temp_df.columns = ["日期"] + year_list
    temp_df = temp_df[:-1]
    temp_df['2015年'] = pd.to_numeric(temp_df['2015年'])
    temp_df['2016年'] = pd.to_numeric(temp_df['2016年'])
    temp_df['2017年'] = pd.to_numeric(temp_df['2017年'])
    temp_df['2018年'] = pd.to_numeric(temp_df['2018年'])
    temp_df['2019年'] = pd.to_numeric(temp_df['2019年'])
    temp_df['2020年'] = pd.to_numeric(temp_df['2020年'])
    temp_df['2021年'] = pd.to_numeric(temp_df['2021年'])
    return temp_df


def futures_egg_price() -> pd.DataFrame:
    """
    2015-2021年鸡蛋价格走势
    https://www.jidan7.com/trend/
    :return: 2015-2021年鸡蛋价格走势
    :rtype: pandas.DataFrame
    """
    url = "https://www.jidan7.com/trend/"
    r = requests.get(url)
    soup = BeautifulSoup(r.text, "lxml")
    js_text = soup.find_all("script")[8].string
    js_text_processed = js_text.replace("\r\n", "")
    re.findall(r"data: (.*)", js_text_processed)
    js_text_processed = re.findall(r"(\[.*?])", js_text_processed)
    date_list = eval(js_text_processed[2])
    value_2015_list = eval(re.findall(r"data: (\[.*?])", js_text_processed[3])[0])
    temp_df = pd.DataFrame(
        [
            date_list,
            value_2015_list,
        ]
    ).T
    temp_df.dropna(how="any", inplace=True)
    temp_df.columns = [
        "date",
        "price",
    ]
    temp_df['price'] = pd.to_numeric(temp_df['price'])
    return temp_df


def futures_egg_price_area() -> pd.DataFrame:
    """
    各主产区鸡蛋均价
    https://www.jidan7.com/trend/
    :return: 各主产区鸡蛋均价
    :rtype: pandas.DataFrame
    """
    url = "https://www.jidan7.com/trend/"
    r = requests.get(url)
    soup = BeautifulSoup(r.text, "lxml")
    js_text = soup.find_all("script")[9].string
    js_text_processed = js_text.replace("\r\n", "")
    js_text_processed = re.findall(r"data: (\[.*?])", js_text_processed)
    area_list = eval(js_text_processed[0])
    date_list = eval(js_text_processed[1])
    value_sd_list = eval(js_text_processed[2])
    value_hn_list = eval(js_text_processed[3])
    value_hb_list = eval(js_text_processed[4])
    value_ln_list = eval(js_text_processed[5])
    value_js_list = eval(js_text_processed[6])
    value_hub_list = eval(js_text_processed[7])
    temp_df = pd.DataFrame(
        [
            date_list,
            value_sd_list,
            value_hn_list,
            value_hb_list,
            value_ln_list,
            value_js_list,
            value_hub_list,
        ]
    ).T
    temp_df.dropna(how="any", inplace=True)
    temp_df.columns = ["日期"] + area_list
    temp_df['山东均价'] = pd.to_numeric(temp_df['山东均价'])
    temp_df['河南均价'] = pd.to_numeric(temp_df['河南均价'])
    temp_df['河北均价'] = pd.to_numeric(temp_df['河北均价'])
    temp_df['辽宁均价'] = pd.to_numeric(temp_df['辽宁均价'])
    temp_df['江苏均价'] = pd.to_numeric(temp_df['江苏均价'])
    temp_df['湖北均价'] = pd.to_numeric(temp_df['湖北均价'])
    return temp_df


if __name__ == "__main__":
    futures_egg_price_yearly_df = futures_egg_price_yearly()
    print(futures_egg_price_yearly_df)

    futures_egg_price_df = futures_egg_price()
    print(futures_egg_price_df)

    futures_egg_price_area_df = futures_egg_price_area()
    print(futures_egg_price_area_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/6/14 12:49
Desc: 新浪财经-中行人民币牌价历史数据查询
http://biz.finance.sina.com.cn/forex/forex.php?startdate=2012-01-01&enddate=2021-06-14&money_code=EUR&type=0
"""
import pandas as pd
import requests
from bs4 import BeautifulSoup
from tqdm import tqdm


def _currency_boc_sina_map(date: str = "20210614") -> dict:
    """
    外汇 symbol 和代码映射
    :param date: 交易日
    :type date: str
    :return: 外汇 symbol 和代码映射
    :rtype: dict
    """
    url = "http://biz.finance.sina.com.cn/forex/forex.php"
    params = {
        "startdate": "2012-01-01",
        "enddate": "-".join([date[:4], date[4:6], date[6:]]),
        "money_code": "EUR",
        "type": "0",
    }
    r = requests.get(url, params=params)
    r.encoding = "gbk"
    soup = BeautifulSoup(r.text, "lxml")
    data_dict = dict(
        zip(
            [
                item.text
                for item in soup.find(attrs={"id": "money_code"}).find_all("option")
            ],
            [
                item["value"]
                for item in soup.find(attrs={"id": "money_code"}).find_all("option")
            ],
        )
    )
    return data_dict


def currency_boc_sina(symbol: str = "美元", date: str = "20210614") -> pd.DataFrame:
    """
    新浪财经-中行人民币牌价历史数据查询
    http://biz.finance.sina.com.cn/forex/forex.php?startdate=2012-01-01&enddate=2021-06-14&money_code=EUR&type=0
    :param symbol: choice of {'美元', '英镑', '欧元', '澳门元', '泰国铢', '菲律宾比索', '港币', '瑞士法郎', '新加坡元', '瑞典克朗', '丹麦克朗', '挪威克朗', '日元', '加拿大元', '澳大利亚元', '新西兰元', '韩国元'}
    :type symbol: str
    :param date: 交易日
    :type date: str
    :return: 中行人民币牌价历史数据查询
    :rtype: pandas.DataFrame
    """
    data_dict = _currency_boc_sina_map(date="20210614")
    url = "http://biz.finance.sina.com.cn/forex/forex.php"
    params = {
        "money_code": data_dict[symbol],
        "type": "0",
        "startdate": "2012-01-01",
        "enddate": "-".join([date[:4], date[4:6], date[6:]]),
        "page": "1",
        "call_type": "ajax",
    }
    r = requests.get(url, params=params)
    soup = BeautifulSoup(r.text, "lxml")
    soup.find(attrs={"id": "money_code"})
    page_num = int(soup.find_all("a", attrs={"class": "page"})[-2].text)
    big_df = pd.DataFrame()
    for page in tqdm(range(1, page_num + 1)):
        params.update({"page": page})
        r = requests.get(url, params=params)
        temp_df = pd.read_html(r.text, header=0)[0]
        big_df = big_df.append(temp_df, ignore_index=True)
    big_df.columns = [
        "日期",
        "中行汇买价",
        "中行钞买价",
        "中行钞卖价/汇卖价",
        "央行中间价",
    ]
    return big_df


if __name__ == "__main__":
    currency_boc_sina_df = currency_boc_sina(symbol="美元", date="20210614")
    print(currency_boc_sina_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/9/11 22:51
Desc: 财联社-今日快讯
https://www.cls.cn/searchPage?keyword=%E5%BF%AB%E8%AE%AF&type=all
"""
import pandas as pd
import requests


def stock_zh_a_alerts_cls() -> pd.DataFrame:
    """
    财联社-今日快讯
    https://www.cls.cn/searchPage?keyword=%E5%BF%AB%E8%AE%AF&type=all
    :return: 财联社-今日快讯
    :rtype: pandas.DataFrame
    """
    url = "https://www.cls.cn/api/sw"
    params = {"app": "CailianpressWeb", "os": "web", "sv": "7.5.5"}
    headers = {
        "Host": "www.cls.cn",
        "Connection": "keep-alive",
        "Content-Length": "112",
        "Accept": "application/json, text/plain, */*",
        "User-Agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/88.0.4324.150 Safari/537.36",
        "Content-Type": "application/json;charset=UTF-8",
        "Origin": "https://www.cls.cn",
        "Sec-Fetch-Site": "same-origin",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Dest": "empty",
        "Accept-Encoding": "gzip, deflate, br",
        "Accept-Language": "zh-CN,zh;q=0.9,en-US;q=0.8,en;q=0.7",
    }
    payload = {
        "type": "telegram",
        "keyword": "快讯",
        "page": 0,
        "rn": 10000,
        "os": "web",
        "sv": "7.2.2",
        "app": "CailianpressWeb",
    }
    r = requests.post(url, headers=headers, params=params, json=payload)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"]["telegram"]["data"])
    temp_df = temp_df[["descr", "time"]]
    temp_df["descr"] = temp_df["descr"].astype(str).str.replace("</em>", "")
    temp_df["descr"] = temp_df["descr"].str.replace("<em>", "")
    temp_df["time"] = pd.to_datetime(temp_df["time"], unit="s").dt.date
    temp_df.columns = ["快讯信息", "时间"]
    temp_df = temp_df[["时间", "快讯信息"]]
    return temp_df


if __name__ == "__main__":
    stock_zh_a_alerts_cls_df = stock_zh_a_alerts_cls()
    print(stock_zh_a_alerts_cls_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/6/30 15:55
Desc: 获取 IT桔子 的死亡公司数据、千里马和独角兽
https://www.itjuzi.com/deathCompany
https://www.itjuzi.com/chollima
https://www.itjuzi.com/unicorn
"""
import pandas as pd
import requests
from tqdm import tqdm


def death_company() -> pd.DataFrame:
    """
    IT桔子-死亡公司名单
    https://www.itjuzi.com/deathCompany
    :return: 死亡公司名单
    :rtype: pandas.DataFrame
    """
    temp_df = pd.read_csv(
        "https://jfds-1252952517.cos.ap-chengdu.myqcloud.com/mssdk/data/data_juzi/juzi.csv"
    )
    for page in tqdm(range(1, 3)):
        url = "https://www.itjuzi.com/api/closure"
        params = {"com_prov": "", "sort": "", "page": page, "keyword": "", "cat_id": ""}
        r = requests.get(
            url, params=params
        )  # 如果需要更新此数据，请增加请求头，请求头中必须包括 Authorization 字段
        data_json = r.json()
        data_df = data_json["data"]["info"]
        data_df = pd.DataFrame(data_df)
        data_df = data_df[
            [
                "com_name",
                "born",
                "com_change_close_date",
                "live_time",
                "total_money",
                "cat_name",
                "com_prov",
            ]
        ]
        temp_df = temp_df.append(data_df, ignore_index=True)
        temp_df.drop_duplicates(inplace=True, ignore_index=True)
    temp_df.reset_index(inplace=True, drop=True)
    temp_df.columns = [
        "公司简称",
        "成立时间",
        "关闭时间",
        "存活天数",
        "融资规模",
        "行业",
        "地点",
    ]
    return temp_df


def nicorn_company() -> pd.DataFrame:
    """
    此数据未更新
    IT桔子-独角兽公司
    https://www.itjuzi.com/unicorn
    :return: 独角兽公司
    :rtype: pandas.DataFrame
    """
    temp_df = pd.read_csv(
        "https://jfds-1252952517.cos.ap-chengdu.myqcloud.com/mssdk/data/data_juzi/nicorn_company.csv",
        index_col=0,
    )
    for i in tqdm(range(1, 2)):
        url = f"https://www.itjuzi.com/api/maxima"
        params = {
            "page": i,
        }
        r = requests.get(url, params=params)
        data_json = r.json()
        data_df = data_json["data"]["data"]
        data_df = pd.DataFrame(data_df)
        temp_df = temp_df.append(data_df, ignore_index=True)
        temp_df.drop_duplicates(inplace=True)
    temp_df.reset_index(drop=True, inplace=True)
    del temp_df['com_id']
    del temp_df['com_logo_archive']
    del temp_df['com_city']
    del temp_df['invse_year']
    del temp_df['invse_month']
    del temp_df['invse_day']
    del temp_df['invse_guess_particulars']
    del temp_df['invse_detail_money']
    del temp_df['invse_currency_id']
    del temp_df['invse_similar_money_id']
    del temp_df['invse_round_id']
    del temp_df['money']
    del temp_df['invse_money']
    del temp_df['round']
    del temp_df['com_scope_id']
    temp_df.reset_index(inplace=True)
    temp_df['index'] = range(1, len(temp_df)+1)
    temp_df.columns = [
        "序号",
        "公司",
        "地区",
        "行业",
        "子行业",
    ]
    return temp_df


def maxima_company() -> pd.DataFrame:
    """
    此数据未更新
    IT桔子-千里马公司
    https://www.itjuzi.com/chollima
    :return: 千里马公司
    :rtype: pandas.DataFrame
    """
    temp_df = pd.read_csv(
        "https://jfds-1252952517.cos.ap-chengdu.myqcloud.com/mssdk/data/data_juzi/maxima.csv",
        index_col=0,
    )
    for i in range(1, 2):
        url = f"https://www.itjuzi.com/api/maxima/"
        params = {
            "page": i,
            "com_prov": "",
            "cat_id": "",
            "order_id": "1",
            "com_name": "",
        }
        r = requests.get(url, params=params)
        data_json = r.json()
        data_df = data_json["data"]["data"]
        data_df = pd.DataFrame(data_df)
        temp_df = temp_df.append(data_df, ignore_index=True)
        temp_df.drop_duplicates(inplace=True)
    temp_df.reset_index(drop=True, inplace=True)
    del temp_df['com_id']
    del temp_df['com_logo_archive']
    del temp_df['com_scope_id']
    del temp_df['invse_year']
    del temp_df['invse_month']
    del temp_df['invse_day']
    del temp_df['invse_similar_money_id']
    del temp_df['invse_guess_particulars']
    del temp_df['invse_detail_money']
    del temp_df['invse_currency_id']
    del temp_df['invse_round_id']
    del temp_df['money']
    del temp_df['invse_money']
    del temp_df['round']
    temp_df.reset_index(inplace=True)
    temp_df['index'] = range(1, len(temp_df)+1)
    temp_df.columns = [
        "序号",
        "公司",
        "行业",
        "地区",
    ]
    return temp_df


if __name__ == "__main__":
    death_company_df = death_company()
    print(death_company_df)

    nicorn_company_df = nicorn_company()
    print(nicorn_company_df)

    maxima_company_df = maxima_company()
    print(maxima_company_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/8/20 18:02
Desc: 东方财富网-数据中心-特色数据-股权质押
东方财富网-数据中心-特色数据-股权质押-股权质押市场概况: http://data.eastmoney.com/gpzy/marketProfile.aspx
东方财富网-数据中心-特色数据-股权质押-上市公司质押比例: http://data.eastmoney.com/gpzy/pledgeRatio.aspx
东方财富网-数据中心-特色数据-股权质押-重要股东股权质押明细: http://data.eastmoney.com/gpzy/pledgeDetail.aspx
东方财富网-数据中心-特色数据-股权质押-质押机构分布统计-证券公司: http://data.eastmoney.com/gpzy/distributeStatistics.aspx
东方财富网-数据中心-特色数据-股权质押-质押机构分布统计-银行: http://data.eastmoney.com/gpzy/distributeStatistics.aspx
东方财富网-数据中心-特色数据-股权质押-行业数据: http://data.eastmoney.com/gpzy/industryData.aspx
"""
import math

from mssdk.utils import demjson
import pandas as pd
import requests
from tqdm import tqdm


def stock_em_gpzy_profile() -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-股权质押-股权质押市场概况
    http://data.eastmoney.com/gpzy/marketProfile.aspx
    :return: 股权质押市场概况
    :rtype: pandas.DataFrame
    """
    url = "http://dcfm.eastmoney.com/EM_MutiSvcExpandInterface/api/js/get"
    params = {
        "type": "ZD_SUM",
        "token": "70f12f2f4f091e459a279469fe49eca5",
        "cmd": "",
        "st": "tdate",
        "sr": "-1",
        "p": "1",
        "ps": "5000",
        "js": "var zvxnZOnT={pages:(tp),data:(x),font:(font)}",
        "rt": "52583914",
    }
    temp_df = pd.DataFrame()
    res = requests.get(url, params=params)
    data_text = res.text
    data_json = demjson.decode(data_text[data_text.find("={") + 1 :])
    map_dict = dict(
        zip(
            pd.DataFrame(data_json["font"]["FontMapping"])["code"],
            pd.DataFrame(data_json["font"]["FontMapping"])["value"],
        )
    )
    for key, value in map_dict.items():
        data_text = data_text.replace(key, str(value))
    data_json = demjson.decode(data_text[data_text.find("={") + 1 :])
    temp_df = temp_df.append(pd.DataFrame(data_json["data"]), ignore_index=True)
    temp_df.columns = [
        "交易日期",
        "sc_zsz",
        "平均质押比例(%)",
        "涨跌幅",
        "A股质押总比例(%)",
        "质押公司数量",
        "质押笔数",
        "质押总股数(股)",
        "质押总市值(元)",
        "沪深300指数",
    ]
    temp_df = temp_df[
        [
            "交易日期",
            "平均质押比例(%)",
            "涨跌幅",
            "A股质押总比例(%)",
            "质押公司数量",
            "质押笔数",
            "质押总股数(股)",
            "质押总市值(元)",
            "沪深300指数",
        ]
    ]
    temp_df["交易日期"] = pd.to_datetime(temp_df["交易日期"])
    return temp_df


def stock_em_gpzy_pledge_ratio(trade_date: str = "2020-08-07") -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-股权质押-上市公司质押比例
    http://data.eastmoney.com/gpzy/pledgeRatio.aspx
    :param trade_date: 指定交易日, 访问 http://data.eastmoney.com/gpzy/pledgeRatio.aspx 查询
    :type trade_date: str
    :return: 上市公司质押比例
    :rtype: pandas.DataFrame
    """
    url = "http://dcfm.eastmoney.com/EM_MutiSvcExpandInterface/api/js/get"
    temp_df = pd.DataFrame()
    params = {
        "type": "ZD_QL_LB",
        "token": "70f12f2f4f091e459a279469fe49eca5",
        "cmd": "",
        "st": "amtshareratio",
        "sr": "-1",
        "p": "1",
        "ps": "5000",
        "js": "var rlJqyOhv={pages:(tp),data:(x),font:(font)}",
        "filter": f"(tdate='{trade_date}')",
        "rt": "52584436",
    }
    res = requests.get(url, params=params)
    data_text = res.text
    data_json = demjson.decode(data_text[data_text.find("={") + 1 :])
    map_dict = dict(
        zip(
            pd.DataFrame(data_json["font"]["FontMapping"])["code"],
            pd.DataFrame(data_json["font"]["FontMapping"])["value"],
        )
    )
    for key, value in map_dict.items():
        data_text = data_text.replace(key, str(value))
    data_json = demjson.decode(data_text[data_text.find("={") + 1 :])
    temp_df = temp_df.append(pd.DataFrame(data_json["data"]), ignore_index=True)
    temp_df.columns = [
        "股票代码",
        "股票简称",
        "交易日期",
        "所属行业",
        "blfb",
        "质押比例(%)",
        "质押股数(股)",
        "质押市值(元)",
        "质押笔数",
        "无限售股质押数(股)",
        "限售股质押数(股)",
        "近一年涨跌幅(%)",
    ]
    temp_df = temp_df[
        [
            "股票代码",
            "股票简称",
            "交易日期",
            "所属行业",
            "质押比例(%)",
            "质押股数(股)",
            "质押市值(元)",
            "质押笔数",
            "无限售股质押数(股)",
            "限售股质押数(股)",
            "近一年涨跌幅(%)",
        ]
    ]
    temp_df["交易日期"] = pd.to_datetime(temp_df["交易日期"])
    return temp_df


def _get_page_num_gpzy_market_pledge_ratio_detail() -> int:
    """
    东方财富网-数据中心-特色数据-股权质押-重要股东股权质押明细
    http://data.eastmoney.com/gpzy/pledgeDetail.aspx
    :return: int 获取 重要股东股权质押明细 的总页数
    """
    url = "http://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        'sortColumns': 'NOTICE_DATE',
        'sortTypes': '-1',
        'pageSize': '500',
        'pageNumber': '1',
        'reportName': 'RPTA_APP_ACCUMDETAILS',
        'columns': 'ALL',
        'quoteColumns': '',
        'source': 'WEB',
        'client': 'WEB',
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = math.ceil(int(data_json['result']['count']) / 500)
    return total_page


def stock_em_gpzy_pledge_ratio_detail() -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-股权质押-重要股东股权质押明细
    http://data.eastmoney.com/gpzy/pledgeDetail.aspx
    :return: pandas.DataFrame
    """
    url = "http://datacenter-web.eastmoney.com/api/data/v1/get"
    page_num = _get_page_num_gpzy_market_pledge_ratio_detail()
    temp_df = pd.DataFrame()
    for page in tqdm(range(1, page_num + 1), leave=False):
        params = {
            'sortColumns': 'NOTICE_DATE',
            'sortTypes': '-1',
            'pageSize': '500',
            'pageNumber': page,
            'reportName': 'RPTA_APP_ACCUMDETAILS',
            'columns': 'ALL',
            'quoteColumns': '',
            'source': 'WEB',
            'client': 'WEB',
        }
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = temp_df.append(pd.DataFrame(data_json['result']["data"]), ignore_index=True)
    temp_df.reset_index(inplace=True)
    temp_df['index'] = range(1, len(temp_df)+1)
    temp_df.columns = [
        "序号",
        "股票简称",
        "_",
        "股票代码",
        "股东名称",
        "_",
        "_",
        "_",
        "公告日期",
        "质押机构",
        "质押股份数量",
        "占所持股份比例",
        "占总股本比例",
        "质押日收盘价",
        "质押开始日期",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "预估平仓线",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "最新价",
        "_",
        "_",
        "_",
        "_",
        "_",
    ]
    temp_df = temp_df[
        [
            "序号",
            "股票代码",
            "股票简称",
            "股东名称",
            "质押股份数量",
            "占所持股份比例",
            "占总股本比例",
            "质押机构",
            "最新价",
            "质押日收盘价",
            "预估平仓线",
            "质押开始日期",
            "公告日期",
        ]
    ]
    temp_df["公告日期"] = pd.to_datetime(temp_df["公告日期"]).dt.date
    temp_df["质押开始日期"] = pd.to_datetime(temp_df["质押开始日期"]).dt.date
    return temp_df


def _get_page_num_gpzy_distribute_statistics_company() -> int:
    """
    东方财富网-数据中心-特色数据-股权质押-质押机构分布统计-证券公司
    http://data.eastmoney.com/gpzy/distributeStatistics.aspx
    :return: int 获取 质押机构分布统计-证券公司 的总页数
    """
    url = "http://dcfm.eastmoney.com/EM_MutiSvcExpandInterface/api/js/get"
    params = {
        "type": "GDZY_ZYJG_SUM",
        "token": "70f12f2f4f091e459a279469fe49eca5",
        "cmd": "",
        "st": "scode_count",
        "sr": "-1",
        "p": "1",
        "ps": "5000",
        "js": "var bLnpEFtJ={pages:(tp),data:(x),font:(font)}",
        "filter": "(hy_name='券商信托')",
        "rt": "52584592",
    }
    res = requests.get(url, params=params)
    data_json = demjson.decode(res.text[res.text.find("={") + 1 :])
    return data_json["pages"]


def stock_em_gpzy_distribute_statistics_company() -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-股权质押-质押机构分布统计-证券公司
    http://data.eastmoney.com/gpzy/distributeStatistics.aspx
    :return: pandas.DataFrame
    """
    url = "http://dcfm.eastmoney.com/EM_MutiSvcExpandInterface/api/js/get"
    page_num = _get_page_num_gpzy_distribute_statistics_company()
    temp_df = pd.DataFrame()
    for page in tqdm(range(1, page_num + 1), leave=True):
        params = {
            "type": "GDZY_ZYJG_SUM",
            "token": "70f12f2f4f091e459a279469fe49eca5",
            "cmd": "",
            "st": "scode_count",
            "sr": "-1",
            "p": str(page),
            "ps": "5000",
            "js": "var bLnpEFtJ={pages:(tp),data:(x),font:(font)}",
            "filter": "(hy_name='券商信托')",
            "rt": "52584592",
        }
        res = requests.get(url, params=params)
        data_text = res.text
        data_json = demjson.decode(data_text[data_text.find("={") + 1 :])
        map_dict = dict(
            zip(
                pd.DataFrame(data_json["font"]["FontMapping"])["code"],
                pd.DataFrame(data_json["font"]["FontMapping"])["value"],
            )
        )
        for key, value in map_dict.items():
            data_text = data_text.replace(key, str(value))
        data_json = demjson.decode(data_text[data_text.find("={") + 1 :])
        temp_df = temp_df.append(pd.DataFrame(data_json["data"]), ignore_index=True)
    temp_df.columns = [
        "质押公司股票代码",
        "_",
        "jg_yjx_type_1",
        "jg_yjx_type_2",
        "质押机构",
        "行业名称",
        "质押公司数量",
        "质押笔数",
        "质押数量(股)",
        "未达预警线比例(%)",
        "达到预警线未达平仓线比例(%)",
        "达到平仓线比例(%)",
    ]
    temp_df = temp_df[
        [
            "质押公司股票代码",
            "质押机构",
            "行业名称",
            "质押公司数量",
            "质押笔数",
            "质押数量(股)",
            "未达预警线比例(%)",
            "达到预警线未达平仓线比例(%)",
            "达到平仓线比例(%)",
        ]
    ]
    return temp_df


def _get_page_num_gpzy_distribute_statistics_bank() -> int:
    """
    东方财富网-数据中心-特色数据-股权质押-质押机构分布统计-银行
    http://data.eastmoney.com/gpzy/distributeStatistics.aspx
    :return: int 获取 质押机构分布统计-银行 的总页数
    """
    url = "http://dcfm.eastmoney.com/EM_MutiSvcExpandInterface/api/js/get"
    params = {
        "type": "GDZY_ZYJG_SUM",
        "token": "70f12f2f4f091e459a279469fe49eca5",
        "cmd": "",
        "st": "scode_count",
        "sr": "-1",
        "p": "1",
        "ps": "5000",
        "js": "var AQxIdDuK={pages:(tp),data:(x),font:(font)}",
        "filter": "(hy_name='银行')",
        "rt": "52584617",
    }
    res = requests.get(url, params=params)
    data_json = demjson.decode(res.text[res.text.find("={") + 1 :])
    return data_json["pages"]


def stock_em_gpzy_distribute_statistics_bank() -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-股权质押-质押机构分布统计-银行
    http://data.eastmoney.com/gpzy/distributeStatistics.aspx
    :return: pandas.DataFrame
    """
    url = "http://dcfm.eastmoney.com/EM_MutiSvcExpandInterface/api/js/get"
    page_num = _get_page_num_gpzy_distribute_statistics_company()
    temp_df = pd.DataFrame()
    for page in range(1, page_num + 1):
        print(f"一共{page_num}页, 正在下载第{page}页")
        params = {
            "type": "GDZY_ZYJG_SUM",
            "token": "70f12f2f4f091e459a279469fe49eca5",
            "cmd": "",
            "st": "scode_count",
            "sr": "-1",
            "p": str(page),
            "ps": "5000",
            "js": "var AQxIdDuK={pages:(tp),data:(x),font:(font)}",
            "filter": "(hy_name='银行')",
            "rt": "52584617",
        }
        res = requests.get(url, params=params)
        data_text = res.text
        data_json = demjson.decode(data_text[data_text.find("={") + 1 :])
        map_dict = dict(
            zip(
                pd.DataFrame(data_json["font"]["FontMapping"])["code"],
                pd.DataFrame(data_json["font"]["FontMapping"])["value"],
            )
        )
        for key, value in map_dict.items():
            data_text = data_text.replace(key, str(value))
        data_json = demjson.decode(data_text[data_text.find("={") + 1 :])
        temp_df = temp_df.append(pd.DataFrame(data_json["data"]), ignore_index=True)
    temp_df.columns = [
        "质押公司股票代码",
        "_",
        "jg_yjx_type_1",
        "jg_yjx_type_2",
        "质押机构",
        "行业名称",
        "质押公司数量",
        "质押笔数",
        "质押数量(股)",
        "未达预警线比例(%)",
        "达到预警线未达平仓线比例(%)",
        "达到平仓线比例(%)",
    ]
    temp_df = temp_df[
        [
            "质押公司股票代码",
            "质押机构",
            "行业名称",
            "质押公司数量",
            "质押笔数",
            "质押数量(股)",
            "未达预警线比例(%)",
            "达到预警线未达平仓线比例(%)",
            "达到平仓线比例(%)",
        ]
    ]
    return temp_df


def _get_page_num_gpzy_industry_data() -> int:
    """
    东方财富网-数据中心-特色数据-股权质押-上市公司质押比例-行业数据
    http://data.eastmoney.com/gpzy/industryData.aspx
    :return: int 获取 上市公司质押比例-行业数据 的总页数
    """
    url = "http://dcfm.eastmoney.com/EM_MutiSvcExpandInterface/api/js/get"
    params = {
        "type": "ZD_HY_SUM",
        "token": "70f12f2f4f091e459a279469fe49eca5",
        "cmd": "",
        "st": "amtshareratio_pj",
        "sr": "-1",
        "p": "1",
        "ps": "5000",
        "js": "var SIqThurI={pages:(tp),data:(x),font:(font)}",
        "rt": "52584617",
    }
    res = requests.get(url, params=params)
    data_json = demjson.decode(res.text[res.text.find("={") + 1 :])
    return data_json["pages"]


def stock_em_gpzy_industry_data() -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-股权质押-上市公司质押比例-行业数据
    http://data.eastmoney.com/gpzy/industryData.aspx
    :return: pandas.DataFrame
    """
    url = "http://dcfm.eastmoney.com/EM_MutiSvcExpandInterface/api/js/get"
    page_num = _get_page_num_gpzy_industry_data()
    temp_df = pd.DataFrame()
    for page in range(1, page_num + 1):
        print(f"一共{page_num}页, 正在下载第{page}页")
        params = {
            "type": "ZD_HY_SUM",
            "token": "70f12f2f4f091e459a279469fe49eca5",
            "cmd": "",
            "st": "amtshareratio_pj",
            "sr": "-1",
            "p": str(page),
            "ps": "5000",
            "js": "var SIqThurI={pages:(tp),data:(x),font:(font)}",
            "rt": "52584617",
        }
        res = requests.get(url, params=params)
        data_text = res.text
        data_json = demjson.decode(data_text[data_text.find("={") + 1 :])
        map_dict = dict(
            zip(
                pd.DataFrame(data_json["font"]["FontMapping"])["code"],
                pd.DataFrame(data_json["font"]["FontMapping"])["value"],
            )
        )
        for key, value in map_dict.items():
            data_text = data_text.replace(key, str(value))
        data_json = demjson.decode(data_text[data_text.find("={") + 1 :])
        temp_df = temp_df.append(pd.DataFrame(data_json["data"]), ignore_index=True)
    temp_df.columns = [
        "统计时间",
        "-",
        "行业",
        "平均质押比例(%)",
        "公司家数",
        "质押总笔数",
        "质押总股本",
        "最新质押市值",
    ]
    temp_df = temp_df[["统计时间", "行业", "平均质押比例(%)", "公司家数", "质押总笔数", "质押总股本", "最新质押市值"]]
    temp_df["统计时间"] = pd.to_datetime(temp_df["统计时间"])
    return temp_df


if __name__ == "__main__":
    stock_em_gpzy_profile_df = stock_em_gpzy_profile()
    print(stock_em_gpzy_profile_df)

    stock_em_gpzy_pledge_ratio_df = stock_em_gpzy_pledge_ratio(trade_date="2021-04-30")
    print(stock_em_gpzy_pledge_ratio_df)

    stock_em_gpzy_pledge_ratio_detail_df = stock_em_gpzy_pledge_ratio_detail()
    print(stock_em_gpzy_pledge_ratio_detail_df)

    stock_em_gpzy_distribute_statistics_company_df = (
        stock_em_gpzy_distribute_statistics_company()
    )
    print(stock_em_gpzy_distribute_statistics_company_df)

    stock_em_gpzy_distribute_statistics_bank_df = stock_em_gpzy_distribute_statistics_bank()
    print(stock_em_gpzy_distribute_statistics_bank_df)

    stock_em_gpzy_industry_data_df = stock_em_gpzy_industry_data()
    print(stock_em_gpzy_industry_data_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/11/30 16:35
Desc: mssdk's pypi info file
"""
import re
import ast

import setuptools

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()


def get_version_string():
    """
    Get the mssdk version number
    :return: version number
    :rtype: str, e.g. '0.6.24'
    """
    with open("mssdk/__init__.py", "rb") as _f:
        version_line = re.search(
            r"__version__\s+=\s+(.*)", _f.read().decode("utf-8")
        ).group(1)
        return str(ast.literal_eval(version_line))


setuptools.setup(
    name="mssdk",
    version=get_version_string(),
    author="MaxsWell",
    author_email="mssdk@mssdk.email.cn",
    license="MIT",
    description="mssdk is an elegant and simple financial data interface library for Python, built for human beings!",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/cdmaxsmart/mssdk",
    packages=setuptools.find_packages(),
    install_requires=[
        "beautifulsoup4>=4.9.1",
        "lxml>=4.2.1",
        "matplotlib>=3.1.1",
        "numpy>=1.15.4",
        "pandas>=0.25",
        "requests>=2.22.0",
        "pillow>=6.2.0",
        "pypinyin>=0.35.0",
        "websocket-client>=0.56.0",
        "html5lib>=1.0.1",
        "xlrd>=1.2.0",
        "urllib3>=1.25.8",
        "tqdm>=4.43.0",
        "openpyxl>=3.0.3",
        "jsonpath>=0.82",
        "tabulate>=0.8.6",
        "decorator>=4.4.2",
        "py_mini_racer>=0.6.0",
        "pyyaml",
        "fastapi[all]",
        "uvicorn[standard]",
        "mysqlclient"
    ],
    package_data={"": ["*.py", "*.json", "*.pk", "*.js"]},
    keywords=[
        "stock",
        "option",
        "futures",
        "fund",
        "bond",
        "index",
        "air",
        "finance",
        "spider",
        "quant",
        "quantitative",
        "investment",
        "trading",
        "algotrading",
        "data",
    ],
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",
)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/12/20 14:52
Desc: 南华期货-商品指数历史走势-价格指数-数值
http://www.nanhua.net/nhzc/varietytrend.html
1000 点开始, 用收益率累计
http://www.nanhua.net/ianalysis/varietyindex/price/A.json?t=1574932974280
"""
import time

import requests
import pandas as pd


def futures_nh_index_symbol_table() -> pd.DataFrame:
    """
    南华期货-南华指数所有品种一览表
    http://www.nanhua.net/ianalysis/varietyindex/price/A.json?t=1574932974280
    :return: 南华指数所有品种一览表
    :rtype: pandas.DataFrame
    """
    url = "http://www.nanhua.net/ianalysis/plate-variety.json"
    r = requests.get(url)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json)
    temp_df['firstday'] = pd.to_datetime(temp_df['firstday']).dt.date
    return temp_df


def futures_nh_price_index(symbol: str = "A") -> pd.DataFrame:
    """
    南华期货-南华指数单品种-价格-所有历史数据
    http://www.nanhua.net/ianalysis/varietyindex/price/A.json?t=1574932974280
    :param symbol: 通过 ak.futures_nh_index_symbol_table() 获取
    :type symbol: str
    :return: 南华期货-南华指数单品种-价格-所有历史数据
    :rtype: pandas.Series
    """
    symbol_df = futures_nh_index_symbol_table()
    if symbol in symbol_df["code"].tolist():
        t = time.time()
        url = f"http://www.nanhua.net/ianalysis/varietyindex/price/{symbol}.json?t={int(round(t * 1000))}"
        r = requests.get(url)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json)
        temp_df.columns = ["date", "value"]
        temp_df['date'] = pd.to_datetime(temp_df["date"], unit='ms').dt.date
        return temp_df


if __name__ == "__main__":
    futures_nh_index_symbol_table_df = futures_nh_index_symbol_table()
    print(futures_nh_index_symbol_table_df)

    futures_nh_price_index_df = futures_nh_price_index(symbol="NHAI")
    print(futures_nh_price_index_df)



# -*- coding:utf-8 -*-
# /usr/bin/env python
"""
Date: 2020/10/19 9:28
Desc: 新浪财经-债券-沪深债券-实时行情数据和历史行情数据
http://vip.stock.finance.sina.com.cn/mkt/#hs_z
"""
import datetime
import re

import demjson
import pandas as pd
import requests
from py_mini_racer import py_mini_racer
from tqdm import tqdm

from mssdk.bond.cons import (
    zh_sina_bond_hs_count_url,
    zh_sina_bond_hs_payload,
    zh_sina_bond_hs_url,
    zh_sina_bond_hs_hist_url,
)
from mssdk.stock.cons import hk_js_decode


def get_zh_bond_hs_page_count() -> int:
    """
    行情中心首页-债券-沪深债券的总页数
    http://vip.stock.finance.sina.com.cn/mkt/#hs_z
    :return: 总页数
    :rtype: int
    """
    params = {
        "node": "hs_z",
    }
    res = requests.get(zh_sina_bond_hs_count_url, params=params)
    page_count = int(re.findall(re.compile(r"\d+"), res.text)[0]) / 80
    if isinstance(page_count, int):
        return page_count
    else:
        return int(page_count) + 1


def bond_zh_hs_spot() -> pd.DataFrame:
    """
    新浪财经-债券-沪深债券的实时行情数据, 大量抓取容易封IP
    http://vip.stock.finance.sina.com.cn/mkt/#hs_z
    :return: 所有沪深债券在当前时刻的实时行情数据
    :rtype: pandas.DataFrame
    """
    big_df = pd.DataFrame()
    page_count = get_zh_bond_hs_page_count()
    zh_sina_bond_hs_payload_copy = zh_sina_bond_hs_payload.copy()
    for page in tqdm(range(1, page_count + 1), leave=False):
        zh_sina_bond_hs_payload_copy.update({"page": page})
        res = requests.get(zh_sina_bond_hs_url, params=zh_sina_bond_hs_payload_copy)
        data_json = demjson.decode(res.text)
        big_df = big_df.append(pd.DataFrame(data_json), ignore_index=True)
    return big_df


def bond_zh_hs_daily(symbol: str = "sh010107") -> pd.DataFrame:
    """
    新浪财经-债券-沪深债券的的历史行情数据, 大量抓取容易封IP
    http://vip.stock.finance.sina.com.cn/mkt/#hs_z
    :param symbol: 沪深债券代码; e.g., sh010107
    :type symbol: str
    :return: 指定沪深债券代码的日 K 线数据
    :rtype: pandas.DataFrame
    """
    res = requests.get(
        zh_sina_bond_hs_hist_url.format(
            symbol, datetime.datetime.now().strftime("%Y_%m_%d")
        )
    )
    js_code = py_mini_racer.MiniRacer()
    js_code.eval(hk_js_decode)
    dict_list = js_code.call(
        "d", res.text.split("=")[1].split(";")[0].replace('"', "")
    )  # 执行js解密代码
    data_df = pd.DataFrame(dict_list)
    data_df.index = pd.to_datetime(data_df["date"])
    del data_df["date"]
    data_df = data_df.astype("float")
    return data_df


if __name__ == "__main__":
    bond_zh_hs_daily_df = bond_zh_hs_daily(symbol="sh010107")
    print(bond_zh_hs_daily_df)
    bond_zh_hs_spot_df = bond_zh_hs_spot()
    print(bond_zh_hs_spot_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2020/10/10 15:58
Desc: 日出和日落数据
https://www.timeanddate.com
"""
import pandas as pd
import pypinyin
import requests


def sunrise_city_list() -> list:
    """
    查询日出与日落数据的城市列表
    :return: 所有可以获取的数据的城市列表
    :rtype: list
    """
    url = "https://www.timeanddate.com/sun/china"
    res = requests.get(url)
    city_list = []
    china_city_one_df = pd.read_html(res.text)[1]
    china_city_two_df = pd.read_html(res.text)[2]
    city_list.extend([item.lower() for item in china_city_one_df.iloc[:, 0].tolist()])
    city_list.extend([item.lower() for item in china_city_one_df.iloc[:, 3].tolist()])
    city_list.extend([item.lower() for item in china_city_two_df.iloc[:, 0].tolist()])
    city_list.extend([item.lower() for item in china_city_two_df.iloc[:, 1].tolist()])
    city_list.extend([item.lower() for item in china_city_two_df.iloc[:, 2].tolist()])
    city_list.extend([item.lower() for item in china_city_two_df.iloc[:, 3].tolist()])
    city_list.extend([item.lower() for item in china_city_two_df.iloc[:, 4][:-2].tolist()])
    return city_list


def sunrise_daily(date: str = "20200428", city: str = "北京") -> pd.DataFrame:
    """
    每日日出日落数据
    https://www.timeanddate.com/sun/china/shaoxing
    :param date: 需要查询的日期, e.g., “20200428”
    :type date: str
    :param city: 需要查询的城市; 注意输入的格式, e.g., "北京", "上海"
    :type city: str
    :return: 返回指定日期指定地区的日出日落数据
    :rtype: pandas.DataFrame
    """
    if pypinyin.slug(city, separator='') in sunrise_city_list():
        year = date[:4]
        month = date[4:6]
        url = f"https://www.timeanddate.com/sun/china/{pypinyin.slug(city, separator='')}?month={month}&year={year}"
        res = requests.get(url)
        table = pd.read_html(res.text, header=2)[1]
        month_df = table.iloc[:-1, ]
        day_df = month_df[month_df.iloc[:, 0].astype(str).str.zfill(2) == date[6:]]
        day_df.index = pd.to_datetime([date] * len(day_df), format="%Y%m%d")
        return day_df
    else:
        return "请输入正确的城市名称"


def sunrise_monthly(date: str = "20190801", city: str = "北京") -> pd.DataFrame:
    """
    每个指定 date 所在月份的每日日出日落数据, 如果当前月份未到月底, 则以预测值填充
    https://www.timeanddate.com/sun/china/shaoxing
    :param date: 需要查询的日期, 这里用来指定 date 所在的月份; e.g., “20200428”
    :type date: str
    :param city: 需要查询的城市; 注意输入的格式, e.g., "北京", "上海"
    :type city: str
    :return: 指定 date 所在月份的每日日出日落数据
    :rtype: pandas.DataFrame
    """
    if pypinyin.slug(city, separator='') in sunrise_city_list():
        year = date[:4]
        month = date[4:6]
        url = f"https://www.timeanddate.com/sun/china/{pypinyin.slug(city, separator='')}?month={month}&year={year}"
        res = requests.get(url)
        table = pd.read_html(res.text, header=2)[1]
        month_df = table.iloc[:-1, ]
        month_df.index = [date[:-2]] * len(month_df)
        return month_df
    else:
        return "请输入正确的城市名称"


if __name__ == "__main__":
    sunrise_daily_df = sunrise_daily(date="20210112", city="北京")
    print(sunrise_daily_df)
    sunrise_monthly_df = sunrise_monthly(date="20210112", city="北京")
    print(sunrise_monthly_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/11/23 20:08
Desc: 新浪财经-国内期货-实时数据获取
http://vip.stock.finance.sina.com.cn/quotes_service/view/qihuohangqing.html#titlePos_3
P.S. 注意采集速度, 容易封禁 IP, 如果不能访问请稍后再试
"""
import json
import time

import pandas as pd
import requests

from mssdk.futures.cons import (
    zh_subscribe_exchange_symbol_url,
    zh_match_main_contract_url,
    zh_match_main_contract_payload,
)
from mssdk.futures.futures_contract_detail import futures_contract_detail
from mssdk.utils import demjson


def zh_subscribe_exchange_symbol(exchange: str = "dce") -> dict:
    """
    交易所具体的可交易品种
    http://vip.stock.finance.sina.com.cn/quotes_service/view/qihuohangqing.html#titlePos_1
    :param exchange: choice of {'czce', 'dce', 'shfe', 'cffex'}
    :type exchange: str
    :return: 交易所具体的可交易品种
    :rtype: dict
    """
    r = requests.get(zh_subscribe_exchange_symbol_url)
    r.encoding = "gbk"
    data_text = r.text
    data_json = demjson.decode(data_text[data_text.find("{"): data_text.find("};") + 1])
    if exchange == "czce":
        data_json["czce"].remove("郑州商品交易所")
        return pd.DataFrame(data_json["czce"])
    if exchange == "dce":
        data_json["dce"].remove("大连商品交易所")
        return pd.DataFrame(data_json["dce"])
    if exchange == "shfe":
        data_json["shfe"].remove("上海期货交易所")
        return pd.DataFrame(data_json["shfe"])
    if exchange == "cffex":
        data_json["cffex"].remove("中国金融期货交易所")
        return pd.DataFrame(data_json["cffex"])


def match_main_contract(exchange: str = "dce") -> str:
    """
    获取主力合约
    :param exchange: choice of {'czce', 'dce', 'shfe', 'cffex'}
    :type exchange: str
    :return: 获取主力合约的字符串
    :rtype: str
    """
    subscribe_exchange_list = []
    exchange_symbol_list = zh_subscribe_exchange_symbol(exchange).iloc[:, 1].tolist()
    for item in exchange_symbol_list:
        # item = 'jhb_qh'
        zh_match_main_contract_payload.update({"node": item})
        res = requests.get(
            zh_match_main_contract_url, params=zh_match_main_contract_payload
        )
        data_json = demjson.decode(res.text)
        data_df = pd.DataFrame(data_json)
        try:
            main_contract = data_df[data_df.iloc[:, 3:].duplicated()]
            print(main_contract["symbol"].values[0])
            subscribe_exchange_list.append(main_contract["symbol"].values[0])
        except:
            if len(data_df) == 1:
                subscribe_exchange_list.append(data_df["symbol"].values[0])
                print(data_df["symbol"].values[0])
            else:
                print(item, "无主力合约")
            continue
    print(f"{exchange}主力合约获取成功")
    return ",".join(["nf_" + item for item in subscribe_exchange_list])


def futures_zh_spot(
    subscribe_list: str = "nf_IF1912,nf_TF1912,nf_IH1912,nf_IC1912",
    market: str = "CF",
    adjust: bool = False,
) -> pd.DataFrame:
    """
    期货的实时行情数据
    :param subscribe_list: 行情的字符串组合
    :type subscribe_list: str
    :param market: CF 为商品期货
    :type market: str
    :param adjust: True or False
    :type adjust: bool
    :return: 期货的实时行情数据
    :rtype: pandas.DataFrame
    """
    url = f"https://hq.sinajs.cn/rn={round(time.time() * 1000)}&list={subscribe_list}"
    res = requests.get(url)
    data_df = pd.DataFrame(
        [
            item.strip().split("=")[1].split(",")
            for item in res.text.split(";")
            if item.strip() != ""
        ]
    )
    data_df.iloc[:, 0] = data_df.iloc[:, 0].str.replace('"', "")
    data_df.iloc[:, -1] = data_df.iloc[:, -1].str.replace('"', "")
    if adjust:
        contract_name_list = [item.split("_")[1] for item in subscribe_list.split(",")]
        contract_min_list = []
        contract_exchange_list = []
        for contract_name in contract_name_list:
            # print(contract_name)
            # contract_name = 'AP2101'
            temp_df = futures_contract_detail(contract=contract_name)
            exchange_name = temp_df[temp_df["item"] == "上市交易所"]["value"].values[0]
            contract_exchange_list.append(exchange_name)
            contract_min = temp_df[temp_df["item"] == "最小变动价位"]["value"].values[0]
            contract_min_list.append(contract_min)
        if market == "CF":
            data_df.columns = [
                "symbol",
                "time",
                "open",
                "high",
                "low",
                "last_close",
                "bid_price",
                "ask_price",
                "current_price",
                "avg_price",
                "last_settle_price",
                "buy_vol",
                "sell_vol",
                "hold",
                "volume",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
            ]
            data_df = data_df[
                [
                    "symbol",
                    "time",
                    "open",
                    "high",
                    "low",
                    "current_price",
                    "bid_price",
                    "ask_price",
                    "buy_vol",
                    "sell_vol",
                    "hold",
                    "volume",
                    "avg_price",
                    "last_close",
                    "last_settle_price",
                ]
            ]
            data_df["exchange"] = contract_exchange_list
            data_df["contract"] = contract_name_list
            data_df["contract_min_change"] = contract_min_list
            return data_df
        else:
            data_df.columns = [
                "open",
                "high",
                "low",
                "current_price",
                "volume",
                "amount",
                "hold",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_" "_",
                "time",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "symbol",
            ]
            data_df = data_df[
                [
                    "symbol",
                    "time",
                    "open",
                    "high",
                    "low",
                    "current_price",
                    "hold",
                    "volume",
                    "amount",
                ]
            ]
            data_df["exchange"] = contract_exchange_list
            data_df["contract"] = contract_name_list
            data_df["contract_min_change"] = contract_min_list
            return data_df
    else:
        if market == "CF":
            data_df.columns = [
                "symbol",
                "time",
                "open",
                "high",
                "low",
                "last_close",
                "bid_price",
                "ask_price",
                "current_price",
                "avg_price",
                "last_settle_price",
                "buy_vol",
                "sell_vol",
                "hold",
                "volume",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
            ]
            data_df = data_df[
                [
                    "symbol",
                    "time",
                    "open",
                    "high",
                    "low",
                    "current_price",
                    "bid_price",
                    "ask_price",
                    "buy_vol",
                    "sell_vol",
                    "hold",
                    "volume",
                    "avg_price",
                    "last_close",
                    "last_settle_price",
                ]
            ]
            return data_df
        else:
            data_df.columns = [
                "open",
                "high",
                "low",
                "current_price",
                "volume",
                "amount",
                "hold",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_" "_",
                "time",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "_",
                "symbol",
            ]
            data_df = data_df[
                [
                    "symbol",
                    "time",
                    "open",
                    "high",
                    "low",
                    "current_price",
                    "hold",
                    "volume",
                    "amount",
                ]
            ]
            return data_df


def futures_zh_minute_sina(symbol: str = "IF2008", period: str = "5") -> pd.DataFrame:
    """
    中国各品种期货分钟频率数据
    http://vip.stock.finance.sina.com.cn/quotes_service/view/qihuohangqing.html#titlePos_3
    :param symbol: 可以通过 match_main_contract(exchange="cffex") 获取, 或者访问网页获取
    :type symbol: str
    :param period: choice of {"1": "1分钟", "5": "5分钟", "15": "15分钟", "30": "30分钟", "60": "60分钟"}
    :type period: str
    :return: 指定 symbol 和 period 的数据
    :rtype: pandas.DataFrame
    """
    url = "https://stock2.finance.sina.com.cn/futures/api/jsonp.php/=/InnerFuturesNewService.getFewMinLine"
    params = {
        "symbol": symbol,
        "type": period,
    }
    r = requests.get(url, params=params)
    temp_df = pd.DataFrame(json.loads(r.text.split("=(")[1].split(");")[0]))
    temp_df.columns = ["datetime", "open", "high", "low", "close", "volume", "hold"]
    temp_df['open'] = pd.to_numeric(temp_df['open'])
    temp_df['high'] = pd.to_numeric(temp_df['high'])
    temp_df['low'] = pd.to_numeric(temp_df['low'])
    temp_df['close'] = pd.to_numeric(temp_df['close'])
    temp_df['volume'] = pd.to_numeric(temp_df['volume'])
    temp_df['hold'] = pd.to_numeric(temp_df['hold'])
    return temp_df


def futures_zh_daily_sina(symbol: str = "V2105") -> pd.DataFrame:
    """
    中国各品种期货日频率数据
    https://finance.sina.com.cn/futures/quotes/V2105.shtml
    :param symbol: 可以通过 match_main_contract(exchange="cffex") 获取, 或者访问网页获取
    :type symbol: str
    :return: 指定 symbol 和 period 的数据
    :rtype: pandas.DataFrame
    """
    date = "20210412"
    url = "https://stock2.finance.sina.com.cn/futures/api/jsonp.php/var%20_V21052021_4_12=/InnerFuturesNewService.getDailyKLine"
    params = {
        "symbol": symbol,
        "type": '_'.join([date[:4], date[4:6], date[6:]]),
    }
    r = requests.get(url, params=params)
    temp_df = pd.DataFrame(json.loads(r.text.split("=(")[1].split(");")[0]))
    temp_df.columns = ["date", "open", "high", "low", "close", "volume", "hold", "settle"]
    temp_df['open'] = pd.to_numeric(temp_df['open'])
    temp_df['high'] = pd.to_numeric(temp_df['high'])
    temp_df['low'] = pd.to_numeric(temp_df['low'])
    temp_df['close'] = pd.to_numeric(temp_df['close'])
    temp_df['volume'] = pd.to_numeric(temp_df['volume'])
    temp_df['hold'] = pd.to_numeric(temp_df['hold'])
    temp_df['settle'] = pd.to_numeric(temp_df['settle'])
    return temp_df


if __name__ == "__main__":
    futures_zh_minute_sina_df = futures_zh_minute_sina(symbol="V2201", period="5")
    print(futures_zh_minute_sina_df)

    futures_zh_daily_sina_df = futures_zh_daily_sina(symbol="LH2109")
    print(futures_zh_daily_sina_df)

    # for num in range(11, 21):
    #     print(num)
    #     for last in range(1, 13):
    #         try:
    #             print(f"V{str(num)+str(last).zfill(2)}")
    #             futures_zh_daily_sina_df = futures_zh_daily_sina(symbol=f"M{str(num)+str(last).zfill(2)}")
    #             print(futures_zh_daily_sina_df)
    #         except:
    #             continue

    print("开始接收实时行情, 每秒刷新一次")
    dce_text = match_main_contract(exchange="dce")
    czce_text = match_main_contract(exchange="czce")
    shfe_text = match_main_contract(exchange="shfe")

    while True:
        data = futures_zh_spot(
            subscribe_list=",".join([dce_text, czce_text, shfe_text]),
            market="CF",
            adjust=True,
        )
        print(data)
        time.sleep(3)

    # 金融期货单独订阅
    cffex_text = match_main_contract(exchange="cffex")

    while True:
        data = futures_zh_spot(subscribe_list=cffex_text, market="FF")
        print(data)
        time.sleep(3)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2020/3/10 16:59
Desc: This is a tool for downloading github user email address.
query url: https://api.github.com/graphql
reference url: https://thedatapack.com/tools/find-github-user-email/
"""
from collections import Counter

import jsonpath
import requests


def tool_github_star_list(owner="jindaxiang", name="mssdk"):
    url = "https://api.github.com/graphql"
    query1 = """
    query GetStars($name: String!, $owner: String!) {
      repository(name: $name, owner: $owner) {
        createdAt
        stargazers(first: 100){
          edges {
            node {
              id
              login
              name
              avatarUrl
              __typename
            }
            starredAt
            __typename
          }
          pageInfo {
            startCursor
            endCursor
            hasNextPage
            __typename
          }
          totalCount
          __typename
        }
        __typename
      }
    }
    """
    query2 = """
    query GetStars($name: String!, $owner: String!, $after: String) {
      repository(name: $name, owner: $owner) {
        createdAt
        stargazers(first: 100, after: $after){
          edges {
            node {
              id
              login
              name
              avatarUrl
              __typename
            }
            starredAt
            __typename
          }
          pageInfo {
            startCursor
            endCursor
            hasNextPage
            __typename
          }
          totalCount
          __typename
        }
        __typename
      }
    }
    """
    headers = {
        "Authorization": "Bearer a4d404b52802435f25903641b58aef482f2d3565",
        "content-type": "application/json",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/80.0.3987.132 Safari/537.36",
    }

    variables = {"owner": owner, "name": name}
    payload = {
        "operationName": "GetStars",
        "variables": variables,
        "query": query1
    }
    r = requests.post(url, json=payload, headers=headers)
    data_json = r.json()
    big_list = []
    name_list = jsonpath.jsonpath(data_json, '$..login')
    next_flag = jsonpath.jsonpath(data_json, '$..endCursor')
    has_next = jsonpath.jsonpath(data_json, '$..hasNextPage')[0]
    big_list.extend(name_list)
    while has_next:
        print(next_flag[0])
        variables.update({"after": next_flag[0]})
        payload.update({"query": query2})
        r = requests.post(url, json=payload, headers=headers)
        data_json = r.json()
        name_list = jsonpath.jsonpath(data_json, '$..login')
        next_flag = jsonpath.jsonpath(data_json, '$..endCursor')
        has_next = jsonpath.jsonpath(data_json, '$..hasNextPage')[0]
        big_list.extend(name_list)
    return big_list


def tool_github_email_address(username="lateautumn4lin"):
    params = {
        "per_page": "100"
    }
    r = requests.get(f"https://api.github.com/users/{username}/events", params=params)
    if r.status_code in (403, 443):
        return None
    if jsonpath.jsonpath(r.json(), '$..email'):
        res = jsonpath.jsonpath(r.json(), '$..email')
        word_counts = Counter(res)
        top_three = word_counts.most_common(1)
        if len(top_three) > 0:
            print(top_three[0][0])
            return top_three[0][0]
    return None


if __name__ == '__main__':
    temp_list = tool_github_star_list(owner="PiotrDabkowski", name="Js2Py")
    print(temp_list)
    tool_github_email_address_df = tool_github_email_address(username="lateautumn4lin")
    print(tool_github_email_address_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2020/4/29 19:51
Desc: 东方财富网-经济数据-银行间拆借利率-配置文件
上海银行同业拆借市场
上海银行同业拆借市场-Shibor人民币
隔夜: http://data.eastmoney.com/shibor/shibor.aspx?m=sh&t=99&d=99221&cu=cny&type=009016&p=2
1周: http://data.eastmoney.com/shibor/shibor.aspx?m=sh&t=99&d=99222&cu=cny&type=009017&p=2
2周: http://data.eastmoney.com/shibor/shibor.aspx?m=sh&t=99&d=99223&cu=cny&type=009018&p=2
1月: http://data.eastmoney.com/shibor/shibor.aspx?m=sh&t=99&d=99224&cu=cny&type=009019&p=2
3月: http://data.eastmoney.com/shibor/shibor.aspx?m=sh&t=99&d=99225&cu=cny&type=009020&p=2
6月: http://data.eastmoney.com/shibor/shibor.aspx?m=sh&t=99&d=99226&cu=cny&type=009021&p=2
9月: http://data.eastmoney.com/shibor/shibor.aspx?m=sh&t=99&d=99227&cu=cny&type=009022&p=2
1年: http://data.eastmoney.com/shibor/shibor.aspx?m=sh&t=99&d=99228&cu=cny&type=009023&p=2

中国银行同业拆借市场
中国银行同业拆借市场-Chibor人民币
隔夜: http://data.eastmoney.com/shibor/shibor.aspx?m=ch&t=98&d=99231&cu=cny&type=009086
1周: http://data.eastmoney.com/shibor/shibor.aspx?m=ch&t=98&d=99232&cu=cny&type=009087
2周: http://data.eastmoney.com/shibor/shibor.aspx?m=ch&t=98&d=99233&cu=cny&type=009088
3周: http://data.eastmoney.com/shibor/shibor.aspx?m=ch&t=98&d=99234&cu=cny&type=009089
1月: http://data.eastmoney.com/shibor/shibor.aspx?m=ch&t=98&d=99235&cu=cny&type=009090
2月: http://data.eastmoney.com/shibor/shibor.aspx?m=ch&t=98&d=99236&cu=cny&type=009091
3月: http://data.eastmoney.com/shibor/shibor.aspx?m=ch&t=98&d=99237&cu=cny&type=009092
4月: http://data.eastmoney.com/shibor/shibor.aspx?m=ch&t=98&d=99238&cu=cny&type=009093
6月: http://data.eastmoney.com/shibor/shibor.aspx?m=ch&t=98&d=99239&cu=cny&type=009094
9月: http://data.eastmoney.com/shibor/shibor.aspx?m=ch&t=98&d=992310&cu=cny&type=009095
1年: http://data.eastmoney.com/shibor/shibor.aspx?m=ch&t=98&d=992311&cu=cny&type=009096

伦敦银行同业拆借市场
伦敦银行同业拆借市场-Libor英镑
隔夜: http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=97&d=99241&cu=gbp&type=009001
1周: http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=97&d=99242&cu=gbp&type=009002
1月: http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=97&d=99243&cu=gbp&type=009004
2月: http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=97&d=99244&cu=gbp&type=009005
3月: http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=97&d=99245&cu=gbp&type=009006
8月: http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=97&d=99246&cu=gbp&type=009011
伦敦银行同业拆借市场-Libor美元
隔夜: http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=96&d=99251&cu=usd&type=009001
1周: http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=96&d=99252&cu=usd&type=009002
1月: http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=96&d=99253&cu=usd&type=009004
2月: http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=96&d=99254&cu=usd&type=009005
3月: http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=96&d=99255&cu=usd&type=009006
8月: http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=96&d=99256&cu=usd&type=009011
伦敦银行同业拆借市场-Libor欧元
隔夜: http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=95&d=99261&cu=eur&type=009001
1周: http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=95&d=99262&cu=eur&type=009002
1月: http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=95&d=99263&cu=eur&type=009004
2月: http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=95&d=99264&cu=eur&type=009005
3月: http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=95&d=99265&cu=eur&type=009006
8月: http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=95&d=99266&cu=eur&type=009011
伦敦银行同业拆借市场-Libor日元
隔夜: http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=94&d=99271&cu=jpy&type=009001
1周: http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=94&d=99272&cu=jpy&type=009002
1月: http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=94&d=99273&cu=jpy&type=009004
2月: http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=94&d=99274&cu=jpy&type=009005
3月: http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=94&d=99275&cu=jpy&type=009006
8月: http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=94&d=99276&cu=jpy&type=009011

欧洲银行同业拆借市场
欧洲银行同业拆借市场-Euribor欧元
1周: http://data.eastmoney.com/shibor/shibor.aspx?m=eu&t=93&d=99281&cu=eur&type=009070
2周: http://data.eastmoney.com/shibor/shibor.aspx?m=eu&t=93&d=99282&cu=eur&type=009071
3周: http://data.eastmoney.com/shibor/shibor.aspx?m=eu&t=93&d=99283&cu=eur&type=009072
1月: http://data.eastmoney.com/shibor/shibor.aspx?m=eu&t=93&d=99284&cu=eur&type=009073
2月: http://data.eastmoney.com/shibor/shibor.aspx?m=eu&t=93&d=99285&cu=eur&type=009074
3月: http://data.eastmoney.com/shibor/shibor.aspx?m=eu&t=93&d=99286&cu=eur&type=009075
4月: http://data.eastmoney.com/shibor/shibor.aspx?m=eu&t=93&d=99287&cu=eur&type=009076
5月: http://data.eastmoney.com/shibor/shibor.aspx?m=eu&t=93&d=99288&cu=eur&type=009077
6月: http://data.eastmoney.com/shibor/shibor.aspx?m=eu&t=93&d=99289&cu=eur&type=009078
7月: http://data.eastmoney.com/shibor/shibor.aspx?m=eu&t=93&d=992810&cu=eur&type=009079
8月: http://data.eastmoney.com/shibor/shibor.aspx?m=eu&t=93&d=992811&cu=eur&type=009080
9月: http://data.eastmoney.com/shibor/shibor.aspx?m=eu&t=93&d=992812&cu=eur&type=009081
10月: http://data.eastmoney.com/shibor/shibor.aspx?m=eu&t=93&d=992813&cu=eur&type=009082
11月: http://data.eastmoney.com/shibor/shibor.aspx?m=eu&t=93&d=992814&cu=eur&type=009083
1年: http://data.eastmoney.com/shibor/shibor.aspx?m=eu&t=93&d=992815&cu=eur&type=009084

香港银行同业拆借市场
香港银行同业拆借市场-Hibor港元
隔夜: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=91&d=99301&cu=hkd&type=009048
1周: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=91&d=99302&cu=hkd&type=009049
2周: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=91&d=99303&cu=hkd&type=009050
1月: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=91&d=99304&cu=hkd&type=009051
2月: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=91&d=99305&cu=hkd&type=009052
3月: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=91&d=99306&cu=hkd&type=009053
4月: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=91&d=99307&cu=hkd&type=009054
5月: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=91&d=99308&cu=hkd&type=009055
6月: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=91&d=99309&cu=hkd&type=009056
7月: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=91&d=993010&cu=hkd&type=009057
8月: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=91&d=993011&cu=hkd&type=009058
9月: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=91&d=993012&cu=hkd&type=009059
10月: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=91&d=993013&cu=hkd&type=009060
11月: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=91&d=993014&cu=hkd&type=009061
1年: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=91&d=993015&cu=hkd&type=009062
香港银行同业拆借市场-Hibor美元
隔夜: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=90&d=99311&cu=usd&type=009048
1周: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=90&d=99312&cu=usd&type=009049
2周: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=90&d=99313&cu=usd&type=009050
1月: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=90&d=99314&cu=usd&type=009051
2月: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=90&d=99315&cu=usd&type=009052
3月: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=90&d=99316&cu=usd&type=009053
4月: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=90&d=99317&cu=usd&type=009054
5月: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=90&d=99318&cu=usd&type=009055
6月: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=90&d=99319&cu=usd&type=009056
7月: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=90&d=993110&cu=usd&type=009057
8月: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=90&d=993111&cu=usd&type=009058
9月: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=90&d=993112&cu=usd&type=009059
10月: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=90&d=993113&cu=usd&type=009060
11月: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=90&d=993114&cu=usd&type=009061
1年: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=90&d=993115&cu=usd&type=009062
香港银行同业拆借市场-Hibor人民币
隔夜: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=89&d=99321&cu=cny&type=009048
1周: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=89&d=99322&cu=cny&type=009049
2周: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=89&d=99323&cu=cny&type=009050
1月: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=89&d=99324&cu=cny&type=009051
2月: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=89&d=99325&cu=cny&type=009052
3月: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=89&d=99326&cu=cny&type=009053
6月: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=89&d=99327&cu=cny&type=009056
1年: http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=89&d=99328&cu=cny&type=009062

新加坡银行同业拆借市场
新加坡银行同业拆借市场-Sibor星元
1月: http://data.eastmoney.com/shibor/shibor.aspx?m=sg&t=88&d=99331&cu=sgd&type=009063
2月: http://data.eastmoney.com/shibor/shibor.aspx?m=sg&t=88&d=99332&cu=sgd&type=009064
3月: http://data.eastmoney.com/shibor/shibor.aspx?m=sg&t=88&d=99333&cu=sgd&type=009065
6月: http://data.eastmoney.com/shibor/shibor.aspx?m=sg&t=88&d=99334&cu=sgd&type=009066
9月: http://data.eastmoney.com/shibor/shibor.aspx?m=sg&t=88&d=99335&cu=sgd&type=009067
1年: http://data.eastmoney.com/shibor/shibor.aspx?m=sg&t=88&d=99336&cu=sgd&type=009068
新加坡银行同业拆借市场-Sibor美元
1月: http://data.eastmoney.com/shibor/shibor.aspx?m=sg&t=87&d=99341&cu=usd&type=009063
2月: http://data.eastmoney.com/shibor/shibor.aspx?m=sg&t=87&d=99342&cu=usd&type=009064
3月: http://data.eastmoney.com/shibor/shibor.aspx?m=sg&t=87&d=99343&cu=usd&type=009065
6月: http://data.eastmoney.com/shibor/shibor.aspx?m=sg&t=87&d=99344&cu=usd&type=009066
9月: http://data.eastmoney.com/shibor/shibor.aspx?m=sg&t=87&d=99345&cu=usd&type=009067
1年: http://data.eastmoney.com/shibor/shibor.aspx?m=sg&t=87&d=99346&cu=usd&type=009068
"""
headers = {
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.129 Safari/537.36"
}

market_symbol_indicator_dict = {
    "上海银行同业拆借市场": {"Shibor人民币": {
        "隔夜": "http://data.eastmoney.com/shibor/shibor.aspx?m=sh&t=99&d=99221&cu=cny&type=009016",
        "1周": "http://data.eastmoney.com/shibor/shibor.aspx?m=sh&t=99&d=99222&cu=cny&type=009017",
        "2周": "http://data.eastmoney.com/shibor/shibor.aspx?m=sh&t=99&d=99223&cu=cny&type=009018",
        "1月": "http://data.eastmoney.com/shibor/shibor.aspx?m=sh&t=99&d=99224&cu=cny&type=009019",
        "3月": "http://data.eastmoney.com/shibor/shibor.aspx?m=sh&t=99&d=99225&cu=cny&type=009020",
        "6月": "http://data.eastmoney.com/shibor/shibor.aspx?m=sh&t=99&d=99226&cu=cny&type=009021",
        "9月": "http://data.eastmoney.com/shibor/shibor.aspx?m=sh&t=99&d=99227&cu=cny&type=009022",
        "1年": "http://data.eastmoney.com/shibor/shibor.aspx?m=sh&t=99&d=99228&cu=cny&type=009023",
    }},

    "中国银行同业拆借市场": {"Chibor人民币": {
        "隔夜": "http://data.eastmoney.com/shibor/shibor.aspx?m=ch&t=98&d=99231&cu=cny&type=009086",
        "1周": "http://data.eastmoney.com/shibor/shibor.aspx?m=ch&t=98&d=99232&cu=cny&type=009087",
        "2周": "http://data.eastmoney.com/shibor/shibor.aspx?m=ch&t=98&d=99233&cu=cny&type=009088",
        "3周": "http://data.eastmoney.com/shibor/shibor.aspx?m=ch&t=98&d=99234&cu=cny&type=009089",
        "1月": "http://data.eastmoney.com/shibor/shibor.aspx?m=ch&t=98&d=99235&cu=cny&type=009090",
        "2月": "http://data.eastmoney.com/shibor/shibor.aspx?m=ch&t=98&d=99236&cu=cny&type=009091",
        "3月": "http://data.eastmoney.com/shibor/shibor.aspx?m=ch&t=98&d=99237&cu=cny&type=009092",
        "4月": "http://data.eastmoney.com/shibor/shibor.aspx?m=ch&t=98&d=99238&cu=cny&type=009093",
        "6月": "http://data.eastmoney.com/shibor/shibor.aspx?m=ch&t=98&d=99239&cu=cny&type=009094",
        "9月": "http://data.eastmoney.com/shibor/shibor.aspx?m=ch&t=98&d=992310&cu=cny&type=009095",
        "1年": "http://data.eastmoney.com/shibor/shibor.aspx?m=ch&t=98&d=992311&cu=cny&type=009096",
    }},

    "伦敦银行同业拆借市场": {"Libor英镑": {
        "隔夜": "http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=97&d=99241&cu=gbp&type=009001",
        "1周": "http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=97&d=99242&cu=gbp&type=009002",
        "1月": "http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=97&d=99243&cu=gbp&type=009004",
        "2月": "http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=97&d=99244&cu=gbp&type=009005",
        "3月": "http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=97&d=99245&cu=gbp&type=009006",
        "8月": "http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=97&d=99246&cu=gbp&type=009011",
    },
        "Libor美元": {
            "隔夜": "http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=96&d=99251&cu=usd&type=009001",
            "1周": "http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=96&d=99252&cu=usd&type=009002",
            "1月": "http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=96&d=99253&cu=usd&type=009004",
            "2月": "http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=96&d=99254&cu=usd&type=009005",
            "3月": "http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=96&d=99255&cu=usd&type=009006",
            "8月": "http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=96&d=99256&cu=usd&type=009011",
        },
        "Libor欧元": {
            "隔夜": "http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=95&d=99261&cu=eur&type=009001",
            "1周": "http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=95&d=99262&cu=eur&type=009002",
            "1月": "http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=95&d=99263&cu=eur&type=009004",
            "2月": "http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=95&d=99264&cu=eur&type=009005",
            "3月": "http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=95&d=99265&cu=eur&type=009006",
            "8月": "http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=95&d=99266&cu=eur&type=009011",
        },
        "Libor日元": {
            "隔夜": "http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=94&d=99271&cu=jpy&type=009001",
            "1周": "http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=94&d=99272&cu=jpy&type=009002",
            "1月": "http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=94&d=99273&cu=jpy&type=009004",
            "2月": "http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=94&d=99274&cu=jpy&type=009005",
            "3月": "http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=94&d=99275&cu=jpy&type=009006",
            "8月": "http://data.eastmoney.com/shibor/shibor.aspx?m=ld&t=94&d=99276&cu=jpy&type=009011",
        }},
    "欧洲银行同业拆借市场": {"Euribor欧元": {
        "1周": "http://data.eastmoney.com/shibor/shibor.aspx?m=eu&t=93&d=99281&cu=eur&type=009070",
        "2周": "http://data.eastmoney.com/shibor/shibor.aspx?m=eu&t=93&d=99282&cu=eur&type=009071",
        "3周": "http://data.eastmoney.com/shibor/shibor.aspx?m=eu&t=93&d=99283&cu=eur&type=009072",
        "1月": "http://data.eastmoney.com/shibor/shibor.aspx?m=eu&t=93&d=99284&cu=eur&type=009073",
        "2月": "http://data.eastmoney.com/shibor/shibor.aspx?m=eu&t=93&d=99285&cu=eur&type=009074",
        "3月": "http://data.eastmoney.com/shibor/shibor.aspx?m=eu&t=93&d=99286&cu=eur&type=009075",
        "4月": "http://data.eastmoney.com/shibor/shibor.aspx?m=eu&t=93&d=99287&cu=eur&type=009076",
        "5月": "http://data.eastmoney.com/shibor/shibor.aspx?m=eu&t=93&d=99288&cu=eur&type=009077",
        "6月": "http://data.eastmoney.com/shibor/shibor.aspx?m=eu&t=93&d=99289&cu=eur&type=009078",
        "7月": "http://data.eastmoney.com/shibor/shibor.aspx?m=eu&t=93&d=992810&cu=eur&type=009079",
        "8月": "http://data.eastmoney.com/shibor/shibor.aspx?m=eu&t=93&d=992811&cu=eur&type=009080",
        "9月": "http://data.eastmoney.com/shibor/shibor.aspx?m=eu&t=93&d=992812&cu=eur&type=009081",
        "10月": "http://data.eastmoney.com/shibor/shibor.aspx?m=eu&t=93&d=992813&cu=eur&type=009082",
        "11月": "http://data.eastmoney.com/shibor/shibor.aspx?m=eu&t=93&d=992814&cu=eur&type=009083",
        "1年": "http://data.eastmoney.com/shibor/shibor.aspx?m=eu&t=93&d=992815&cu=eur&type=009084",
    }},
    "香港银行同业拆借市场": {"Hibor港元": {
        "隔夜": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=91&d=99301&cu=hkd&type=009048",
        "1周": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=91&d=99302&cu=hkd&type=009049",
        "2周": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=91&d=99303&cu=hkd&type=009050",
        "1月": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=91&d=99304&cu=hkd&type=009051",
        "2月": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=91&d=99305&cu=hkd&type=009052",
        "3月": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=91&d=99306&cu=hkd&type=009053",
        "4月": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=91&d=99307&cu=hkd&type=009054",
        "5月": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=91&d=99308&cu=hkd&type=009055",
        "6月": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=91&d=99309&cu=hkd&type=009056",
        "7月": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=91&d=993010&cu=hkd&type=009057",
        "8月": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=91&d=993011&cu=hkd&type=009058",
        "9月": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=91&d=993012&cu=hkd&type=009059",
        "10月": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=91&d=993013&cu=hkd&type=009060",
        "11月": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=91&d=993014&cu=hkd&type=009061",
        "1年": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=91&d=993015&cu=hkd&type=009062",
    },
        "Hibor美元": {
            "隔夜": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=90&d=99311&cu=usd&type=009048",
            "1周": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=90&d=99312&cu=usd&type=009049",
            "2周": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=90&d=99313&cu=usd&type=009050",
            "1月": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=90&d=99314&cu=usd&type=009051",
            "2月": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=90&d=99315&cu=usd&type=009052",
            "3月": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=90&d=99316&cu=usd&type=009053",
            "4月": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=90&d=99317&cu=usd&type=009054",
            "5月": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=90&d=99318&cu=usd&type=009055",
            "6月": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=90&d=99319&cu=usd&type=009056",
            "7月": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=90&d=993110&cu=usd&type=009057",
            "8月": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=90&d=993111&cu=usd&type=009058",
            "9月": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=90&d=993112&cu=usd&type=009059",
            "10月": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=90&d=993113&cu=usd&type=009060",
            "11月": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=90&d=993114&cu=usd&type=009061",
            "1年": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=90&d=993115&cu=usd&type=009062",
        },
        "Hibor人民币": {
            "隔夜": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=89&d=99321&cu=cny&type=009048",
            "1周": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=89&d=99322&cu=cny&type=009049",
            "2周": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=89&d=99323&cu=cny&type=009050",
            "1月": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=89&d=99324&cu=cny&type=009051",
            "2月": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=89&d=99325&cu=cny&type=009052",
            "3月": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=89&d=99326&cu=cny&type=009053",
            "6月": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=89&d=99327&cu=cny&type=009056",
            "1年": "http://data.eastmoney.com/shibor/shibor.aspx?m=hk&t=89&d=99328&cu=cny&type=009062",
        }},
    "新加坡银行同业拆借市场": {"Sibor星元": {
        "1月": "http://data.eastmoney.com/shibor/shibor.aspx?m=sg&t=88&d=99331&cu=sgd&type=009063",
        "2月": "http://data.eastmoney.com/shibor/shibor.aspx?m=sg&t=88&d=99332&cu=sgd&type=009064",
        "3月": "http://data.eastmoney.com/shibor/shibor.aspx?m=sg&t=88&d=99333&cu=sgd&type=009065",
        "6月": "http://data.eastmoney.com/shibor/shibor.aspx?m=sg&t=88&d=99334&cu=sgd&type=009066",
        "9月": "http://data.eastmoney.com/shibor/shibor.aspx?m=sg&t=88&d=99335&cu=sgd&type=009067",
        "1年": "http://data.eastmoney.com/shibor/shibor.aspx?m=sg&t=88&d=99336&cu=sgd&type=009068",
    },
        "Sibor美元": {
            "1月": "http://data.eastmoney.com/shibor/shibor.aspx?m=sg&t=87&d=99341&cu=usd&type=009063",
            "2月": "http://data.eastmoney.com/shibor/shibor.aspx?m=sg&t=87&d=99342&cu=usd&type=009064",
            "3月": "http://data.eastmoney.com/shibor/shibor.aspx?m=sg&t=87&d=99343&cu=usd&type=009065",
            "6月": "http://data.eastmoney.com/shibor/shibor.aspx?m=sg&t=87&d=99344&cu=usd&type=009066",
            "9月": "http://data.eastmoney.com/shibor/shibor.aspx?m=sg&t=87&d=99345&cu=usd&type=009067",
            "1年": "http://data.eastmoney.com/shibor/shibor.aspx?m=sg&t=87&d=99346&cu=usd&type=009068",
        }}}



class ResponseError(Exception):
    """Something was wrong with the response from Google"""

    def __init__(self, message, response):
        super(Exception, self).__init__(message)

        # pass response so it can be handled upstream
        self.response = response



# -*- coding:utf-8 -*-
# /usr/bin/env python
"""
Date: 2020/12/28 16:01
Desc: 美国股市(S&P 500)的指数及其收益率，还有常用的14个经济指标
http://www.hec.unil.ch/agoyal/
http://www.hec.unil.ch/agoyal/docs/PredictorData2018.xlsx
"""
import pandas as pd


def agoyal_stock_return(indicator: str = "monthly"):
    """
    This data will be updated every year at May
    :param indicator: monthly, quarterly or annual
    :type indicator: str
    :return: return data at specific indicator
    :rtype: pandas.DataFrame
    """
    url = "http://www.hec.unil.ch/agoyal/docs/PredictorData2018.xlsx"
    if indicator == "monthly":
        return pd.read_excel(url, sheet_name="Monthly", engine="openpyxl")
    if indicator == "quarterly":
        return pd.read_excel(url, sheet_name="Quarterly", engine="openpyxl")
    if indicator == "annual":
        return pd.read_excel(url, sheet_name="Annual", engine="openpyxl")


if __name__ == '__main__':
    agoyal_stock_return_df = agoyal_stock_return(indicator="monthly")
    print(agoyal_stock_return_df)
